@@ -513,8 +513,11 @@ dynany_get_value (DynAny *dynany, CORBA_Environment *ev)
 			int i;
 			/* FIXME: could cache for efficiency */
 			value = any->_value;
-			for (i = 0; i < dynany->idx; i++)
+			value = ALIGN_ADDRESS (value, tc->subtypes [0]->c_align);
+			for (i = 0; i < dynany->idx; i++) {
 				value = DYNANY_GET_OFFSET (value, 1, tc->subtypes [i]);
+				value = ALIGN_ADDRESS (value, tc->subtypes [i+1]->c_align);
+			}
 			break;
 		}
 
@@ -884,6 +887,7 @@ DynamicAny_DynAny_insert_##apiname (DynamicAny_DynAny       obj,		\
 			       CORBA_Environment *ev)				\
 {										\
 	DynAny  *dynany;							\
+	struct { CORBA_##ctype value; } copy_of_value = {value};		\
 										\
 	o_return_if_fail (obj != NULL);						\
 										\
@@ -894,7 +898,7 @@ DynamicAny_DynAny_insert_##apiname (DynamicAny_DynAny       obj,		\
 	if (dynany_type_mismatch (dynany, TC_CORBA_##ctype, ev))		\
 		return;								\
 										\
-	dynany_insert (dynany, TC_CORBA_##ctype, &value, ev);		       	\
+	dynany_insert (dynany, TC_CORBA_##ctype, &copy_of_value.value, ev);	\
 										\
 	return;									\
 }
@@ -926,16 +930,16 @@ DynamicAny_DynAny_get_##apiname (DynamicAny_DynAny  obj,			\
 	DynAny       *dynany;							\
 	CORBA_##ctype value;							\
 										\
-	o_return_val_if_fail (obj != NULL, 0);				       	\
+	o_return_val_if_fail (obj != NULL, (CORBA_##ctype)0);				       	\
 										\
 	dynany = GET_DYNANY (obj);						\
 										\
 	/* FIXME: assumes NULL == CORBA_OBJECT_NIL for simplicity */		\
 	b_return_val_if_fail (dynany != NULL &&					\
-			      dynany->any != NULL, 0);				\
+			      dynany->any != NULL, (CORBA_##ctype)0);				\
 										\
 	if (dynany_type_mismatch (dynany, tc, ev))				\
-		return 0;	       						\
+		return (CORBA_##ctype)0;	       						\
 										\
 	dynany_get (dynany, &value, tc, ev);					\
 										\
@@ -1379,6 +1383,7 @@ DynamicAny_DynStruct_get_members (DynamicAny_DynStruct obj,
 {
 	DynAny *dynany;
 	int i;
+	guint offset = 0;
 	gconstpointer src;
 	DynamicAny_NameValuePairSeq *retval;
 	CORBA_TypeCode tc;
@@ -1408,6 +1413,7 @@ DynamicAny_DynStruct_get_members (DynamicAny_DynStruct obj,
 		CORBA_any      *any;
 		CORBA_TypeCode  subtc = tc->subtypes [i];
 		gpointer        to;
+		gconstpointer   tmpsrc;
 
 		retval->_buffer [i].id = CORBA_string_dup (tc->subnames [i]);
 		any = &retval->_buffer [i].value;
@@ -1415,8 +1421,10 @@ DynamicAny_DynStruct_get_members (DynamicAny_DynStruct obj,
 		any->_type = (CORBA_TypeCode) CORBA_Object_duplicate (
 			(CORBA_Object) subtc, ev);
 		to = any->_value = ORBit_alloc_by_tc (subtc);
-		
-		ORBit_copy_value_core (&src, &to, subtc);
+		offset = ALIGN_VALUE (offset, subtc->c_align);
+		tmpsrc = src + offset;
+		ORBit_copy_value_core (&tmpsrc, &to, subtc);
+		offset += ORBit_gather_alloc_info (subtc);
 	}
 
 	return retval;
@@ -1429,6 +1437,7 @@ DynamicAny_DynStruct_set_members (DynamicAny_DynStruct               obj,
 {
 	DynAny *dynany;
 	int i;
+	guint offset = 0;
 	gpointer dest;
 	CORBA_TypeCode tc;
 
@@ -1473,8 +1482,13 @@ DynamicAny_DynStruct_set_members (DynamicAny_DynStruct               obj,
 	for (i = 0; i < value->_length; i++) {
 		DynamicAny_NameValuePair current = value->_buffer [i];
 		gconstpointer src = current.value._value;
+		CORBA_TypeCode subtc = tc->subtypes [i];
+		gpointer tmpdest;
 
-		ORBit_copy_value_core (&src, &dest, tc->subtypes [i]);
+		offset = ALIGN_VALUE (offset, subtc->c_align);
+		tmpdest = dest + offset;
+		ORBit_copy_value_core (&src, &tmpdest, subtc);
+		offset += ORBit_gather_alloc_info (subtc);
 	}
 }
 
@@ -1514,6 +1528,7 @@ DynamicAny_DynStruct_get_members_as_dyn_any (DynamicAny_DynStruct obj,
 
 		retval->_buffer [i].id = CORBA_string_dup (tc->subnames [i]);
 
+		/* FIXME: this looks broken to me - why does src not advance ? */
 		retval->_buffer [i].value = dynany_create (subtc, src, dynany, ev);
 	}
 
@@ -1671,12 +1686,12 @@ DynamicAny_DynUnion_member_name (DynamicAny_DynUnion obj,
 	DynAny *dynany;
 	CORBA_TypeCode tc;
 	
-	o_return_val_if_fail (obj != NULL, 0);
+	o_return_val_if_fail (obj != NULL, NULL);
 	
 	dynany = GET_DYNANY (obj);
 	b_return_val_if_fail (dynany != NULL &&
 			      dynany->any != NULL &&
-			      dynany->any->_type != NULL, 0);
+			      dynany->any->_type != NULL, NULL);
 
 	tc = dynany->any->_type;
 
