@@ -4,6 +4,7 @@
 #include <string.h>
 
 #include <orbit/orbit.h>
+#include <orbit/orb-core/orb-types.h>
 
 #include "../orbit-init.h"
 #include "../poa/orbit-poa-export.h"
@@ -22,24 +23,40 @@ extern const ORBit_option orbit_supported_options[];
 OrbitDebugFlags _orbit_debug_flags = ORBIT_DEBUG_NONE;
 #endif
 
+#ifdef G_OS_WIN32
+/* Silence gcc warning about no prototype */
+BOOL WINAPI DllMain (HINSTANCE hinstDLL,
+		     DWORD     fdwReason,
+		     LPVOID    lpvReserved);
+#endif
+
 /*
  * Command line option handling.
  */
+#ifdef G_OS_WIN32
+static gboolean     orbit_use_ipv4           = TRUE;
+#else
 static gboolean     orbit_use_ipv4           = FALSE;
+#endif
 static gboolean     orbit_use_ipv6           = FALSE; 
+#ifdef G_OS_WIN32
+static gboolean     orbit_use_usocks         = FALSE;
+#else
 static gboolean     orbit_use_usocks         = TRUE;
+#endif
 static gint         orbit_initial_recv_limit = -1;
 static gboolean     orbit_use_irda           = FALSE;
 static gboolean     orbit_use_ssl            = FALSE;
 static gboolean     orbit_use_genuid_simple  = FALSE;
 static gboolean     orbit_local_only         = FALSE;
+static char        *orbit_net_id             = NULL;
 static gboolean     orbit_use_http_iors      = FALSE;
 static char        *orbit_ipsock             = NULL;
 static char        *orbit_ipname             = NULL;
 static char        *orbit_debug_options      = NULL;
 static char        *orbit_naming_ref         = NULL;
 static GSList      *orbit_initref_list       = NULL; 
-static GSList      *orbit_use_corbaloc       = FALSE;
+static gboolean     orbit_use_corbaloc       = FALSE;
 
 void
 ORBit_ORB_start_servers (CORBA_ORB orb)
@@ -57,6 +74,32 @@ ORBit_ORB_start_servers (CORBA_ORB orb)
 	if (orbit_local_only)
 		create_options |= LINK_CONNECTION_LOCAL_ONLY;
 
+	if (orbit_local_only || (orbit_use_usocks && !(orbit_use_ipv4 || orbit_use_ipv6 || orbit_use_irda || orbit_use_ssl)))
+			link_use_local_hostname (LINK_NET_ID_IS_LOCAL);
+	else {
+		do {
+			if (!orbit_net_id)
+				break;
+
+			if (!strcmp(orbit_net_id, "local")) {
+				link_use_local_hostname (LINK_NET_ID_IS_LOCAL);
+				break;
+			}
+			if (!strcmp(orbit_net_id, "short")) {
+				link_use_local_hostname (LINK_NET_ID_IS_SHORT_HOSTNAME);
+				break;
+			}
+			if (!strcmp(orbit_net_id, "fqdn")) {
+				link_use_local_hostname (LINK_NET_ID_IS_FQDN);
+				break;
+			}
+			if (!strcmp(orbit_net_id, "ipaddr")) {
+				link_use_local_hostname (LINK_NET_ID_IS_IPADDR);
+				break;
+			}
+		} while (0);
+	}
+		    
 	for (info = link_protocol_all (); info->name; info++) {
 		GIOPServer           *server;
 
@@ -102,8 +145,11 @@ static void
 strip_object_profiles (gpointer o, gpointer b, gpointer c)
 {
 	CORBA_Object obj = o;
+
 	IOP_delete_profiles (obj->orb, &obj->profile_list);
 	IOP_delete_profiles (obj->orb, &obj->forward_locations);
+
+	obj->orb = NULL;
 }
 
 static void
@@ -232,6 +278,7 @@ ORBit_setup_debug_flags (void)
 
 static CORBA_ORB _ORBit_orb = NULL;
 static gulong    init_level = 0;
+static gboolean  atexit_shutdown = FALSE;
 
 /*
  *   This is neccessary to clean up any remaining UDS
@@ -247,6 +294,7 @@ shutdown_orb (void)
 		return;
 
 	init_level = 1; /* clobber it */
+	atexit_shutdown = TRUE;
 	
 	CORBA_exception_init (&ev);
 
@@ -254,6 +302,8 @@ shutdown_orb (void)
 	ORBit_RootObject_release (orb);
 
 	CORBA_exception_free (&ev);
+
+	atexit_shutdown = FALSE;
 }
 
 static
@@ -290,6 +340,7 @@ ORBit_initial_references_by_user (CORBA_ORB          orb,
 			 * IDL:omg.org/CosNaming/NamingContext using _is_a()
 			 * operation */ 
 			ORBit_set_initial_reference (orb, "NameService", objref);
+			ORBit_RootObject_release (objref);
 		}
 	}
 
@@ -309,13 +360,16 @@ ORBit_initial_references_by_user (CORBA_ORB          orb,
 			g_warning ("Option ORBInitRef has invalid object reference: %s=%s",  
 				   tuple->key, tuple->value);
 			CORBA_exception_free (ev);
-		}  else if (ORBit_initial_reference_protected_id(tuple->key)) {
-                        g_warning ("Option ORBInitRef permission denied: %s=%s",
-
-                                   tuple->key, tuple->value);
-                } else {
-                        ORBit_set_initial_reference (orb, tuple->key, objref);
-                }
+		} else {
+			if (ORBit_initial_reference_protected_id(tuple->key)) {
+				g_warning ("Option ORBInitRef permission denied: %s=%s",
+					   tuple->key, tuple->value);
+			} else {
+				ORBit_set_initial_reference (orb, tuple->key, objref);
+			}
+			
+			ORBit_RootObject_release (objref);
+		}
 	}
 
 }
@@ -325,7 +379,7 @@ CORBA_ORB_init (int *argc, char **argv,
 		CORBA_ORBid orb_identifier,
 		CORBA_Environment *ev)
 {
-	gboolean threaded;
+	gboolean thread_safe;
 	CORBA_ORB retval;
 	static ORBit_RootObject_Interface orb_if = {
 		ORBIT_ROT_ORB,
@@ -342,9 +396,9 @@ CORBA_ORB_init (int *argc, char **argv,
 
 	if (orb_identifier &&
 	    strstr (orb_identifier, "orbit-local-non-threaded-orb") != NULL)
-		threaded = FALSE;
+		thread_safe = FALSE;
 	else
-		threaded = TRUE;
+		thread_safe = TRUE;
 
 	ORBit_option_parse (argc, argv, orbit_supported_options);
 	
@@ -353,15 +407,20 @@ CORBA_ORB_init (int *argc, char **argv,
 
 	if (_orbit_debug_flags & ORBIT_DEBUG_FORCE_THREADED) {
 		g_warning ("-- Forced orb into threaded mode --");
-		threaded |= TRUE;
+		thread_safe |= TRUE;
 	}
 #endif /* G_ENABLE_DEBUG */
 
 
 	giop_recv_set_limit (orbit_initial_recv_limit);
-	giop_init (threaded,
+	giop_init (thread_safe,
 		   orbit_use_ipv4 || orbit_use_ipv6 ||
 		   orbit_use_irda || orbit_use_ssl);
+
+	if (orb_identifier && thread_safe &&
+	    strstr (orb_identifier, "orbit-io-thread") != NULL)
+		link_set_io_thread (TRUE);
+
 	genuid_init ();
 	_ORBit_object_init ();
 	ORBit_poa_init ();
@@ -379,6 +438,13 @@ CORBA_ORB_init (int *argc, char **argv,
 	retval->default_giop_version = GIOP_LATEST;
 
 	retval->adaptors = g_ptr_array_new ();
+
+	/* init the forward bind hashtable*/
+	retval->forw_binds = g_hash_table_new_full (
+		g_str_hash, g_str_equal,
+		g_free,
+		NULL); 
+
 	ORBit_init_internals (retval, ev);
 	/* FIXME, handle exceptions */ 
 
@@ -410,9 +476,13 @@ CORBA_ORB_object_to_string (CORBA_ORB          orb,
 		return NULL;
 	}
 
-	/* FIXME, do nice integration */ 
 	if (orbit_use_corbaloc) {
-		return ORBit_object_to_corbaloc (obj, ev);
+		out = ORBit_object_to_corbaloc (obj, ev);
+		if (ev->_major == CORBA_NO_EXCEPTION)
+			return out;
+
+		CORBA_exception_free (ev);	
+		/* fall thru, common marshalling */ 
 	}
 	
 	buf = giop_send_buffer_use (orb->default_giop_version);
@@ -544,13 +614,13 @@ CORBA_ORB_create_list (CORBA_ORB          obj,
 		       CORBA_NVList      *new_list,
 		       CORBA_Environment *ev)
 {
-	CORBA_NVList new;
+	CORBA_NVList nvlist;
 
-	new = g_new0 (struct CORBA_NVList_type, 1);
+	nvlist = g_new0 (struct CORBA_NVList_type, 1);
 
-	new->list = g_array_new (FALSE, TRUE, sizeof (CORBA_NamedValue));
+	nvlist->list = g_array_new (FALSE, TRUE, sizeof (CORBA_NamedValue));
 
-	*new_list = new;
+	*new_list = nvlist;
 }
 
 void
@@ -697,13 +767,14 @@ CORBA_ORB_resolve_initial_references (CORBA_ORB          orb,
 		return CORBA_OBJECT_NIL;
 	
 	return ORBit_RootObject_duplicate (objref);
-
+#if 0
  raise_invalid_name:
 	CORBA_exception_set (ev,
 			     CORBA_USER_EXCEPTION,
 			     ex_CORBA_ORB_InvalidName,
 			     NULL);
 	return CORBA_OBJECT_NIL;
+#endif
 }
 
 static CORBA_TypeCode
@@ -1157,9 +1228,10 @@ CORBA_ORB_destroy (CORBA_ORB          orb,
 	if (root_poa &&
 	    ((ORBit_RootObject) root_poa)->refs != 1) {
 #ifdef G_ENABLE_DEBUG
-		g_warning ("CORBA_ORB_destroy: Application still has %d "
-			   "refs to RootPOA.",
-			   ((ORBit_RootObject)root_poa)->refs - 1);
+		if (!atexit_shutdown)
+			g_warning ("CORBA_ORB_destroy: Application still has %d "
+				   "refs to RootPOA.",
+				   ((ORBit_RootObject)root_poa)->refs - 1);
 #endif
 		CORBA_exception_set_system (
 			ev, ex_CORBA_FREE_MEM, CORBA_COMPLETED_NO);
@@ -1188,7 +1260,9 @@ CORBA_ORB_destroy (CORBA_ORB          orb,
 
 		if (leaked_adaptors) {
 #ifdef G_ENABLE_DEBUG
-			g_warning ("CORBA_ORB_destroy: leaked '%d' Object Adaptors", leaked_adaptors);
+			if (!atexit_shutdown)
+				g_warning ("CORBA_ORB_destroy: leaked '%d' Object Adaptors",
+					   leaked_adaptors);
 #endif
 			CORBA_exception_set_system (
 				ev, ex_CORBA_FREE_MEM, CORBA_COMPLETED_NO);
@@ -1196,24 +1270,34 @@ CORBA_ORB_destroy (CORBA_ORB          orb,
 
 		if (((ORBit_RootObject)orb)->refs != 2 + leaked_adaptors) {
 #ifdef G_ENABLE_DEBUG
-			if (((ORBit_RootObject)orb)->refs == 1 + leaked_adaptors)
-				g_warning ("CORBA_ORB_destroy: ORB unreffed but not _destroy'd");
-			else
-				g_warning ("CORBA_ORB_destroy: ORB still has %d refs.",
-					   ((ORBit_RootObject)orb)->refs - 1 - leaked_adaptors);
+			if (!atexit_shutdown) {
+				if (((ORBit_RootObject)orb)->refs == 1 + leaked_adaptors)
+					g_warning ("CORBA_ORB_destroy: ORB unreffed but not _destroy'd");
+				else
+					g_warning ("CORBA_ORB_destroy: ORB still has %d refs.",
+						   ((ORBit_RootObject)orb)->refs - 1 - leaked_adaptors);
+			}
 #endif
 			CORBA_exception_set_system (
 				ev, ex_CORBA_FREE_MEM, CORBA_COMPLETED_NO);
 		}
 	}
 
+	/* destroy the forward bind hashtable*/
+	g_hash_table_destroy (orb->forw_binds);
+	
 	orb->life_flags |= ORBit_LifeF_Destroyed;
 
+	if (orb->lock) {
+		g_mutex_free (orb->lock);
+		orb->lock = NULL;
+	}
+
 	ORBit_RootObject_release (orb);
 
 	/* At this stage there should be 1 ref left in the system -
 	 * on the ORB */
-	if (ORBit_RootObject_shutdown ())
+	if (ORBit_RootObject_shutdown (!atexit_shutdown))
 		CORBA_exception_set_system (
 			ev, ex_CORBA_FREE_MEM, CORBA_COMPLETED_NO);
 }
@@ -1256,13 +1340,15 @@ ORBit_set_initial_reference (CORBA_ORB  orb,
 			     gchar     *identifier,
 			     gpointer   objref)
 {
+	CORBA_Object old_objref;
+
 	if (!orb->initial_refs)
 		orb->initial_refs = g_hash_table_new (g_str_hash, g_str_equal);
 
-	if (g_hash_table_lookup (orb->initial_refs, identifier))
-               /* FIXME, release object before removing from hash-table,
-                   Frank 2003/06/22 */
+	if ((old_objref = g_hash_table_lookup (orb->initial_refs, identifier))) {
+		ORBit_RootObject_release (old_objref);
 		g_hash_table_remove (orb->initial_refs, identifier);
+	}
 
 	g_hash_table_insert (orb->initial_refs,
 			     identifier,
@@ -1275,7 +1361,13 @@ ORBit_ORB_forw_bind (CORBA_ORB                   orb,
 		     CORBA_Object                obj,
 		     CORBA_Environment          *ev)
 {
-	g_warning ("ORBit_ORB_forw_bind NYI");
+	
+	if (obj)
+		g_hash_table_insert (orb->forw_binds, objkey->_buffer, obj);
+	else {
+		g_hash_table_remove(orb->forw_binds, objkey->_buffer);
+	}
+	
 }
 
 gboolean
@@ -1303,6 +1395,7 @@ const ORBit_option orbit_supported_options[] = {
  	{ "ORBIIOPIPSock",      ORBIT_OPTION_STRING,  &orbit_ipsock },
 	{ "ORBInitialMsgLimit", ORBIT_OPTION_INT,     &orbit_initial_recv_limit },
 	{ "ORBLocalOnly",       ORBIT_OPTION_BOOLEAN, &orbit_local_only },
+ 	{ "ORBNetID",           ORBIT_OPTION_STRING,  &orbit_net_id },
 	/* warning: this option is a security risk unless used with LocalOnly */
 	{ "ORBIIOPIPv4",        ORBIT_OPTION_BOOLEAN, &orbit_use_ipv4 },
 	{ "ORBIIOPIPv6",        ORBIT_OPTION_BOOLEAN, &orbit_use_ipv6 },
@@ -1319,3 +1412,69 @@ const ORBit_option orbit_supported_options[] = {
 	{ "ORBCorbaloc",        ORBIT_OPTION_BOOLEAN, &orbit_use_corbaloc},
 	{ NULL,                 0,                    NULL },
 };
+
+#ifdef G_OS_WIN32
+
+const gchar *ORBit_win32_typelib_dir;
+const gchar *ORBit_win32_system_rcfile;
+
+/* DllMain function needed to fetch the DLL name and deduce the
+ * installation directory from that, and then form the pathnames for
+ * the typelib directory and system orbitrc file.
+ */
+BOOL WINAPI
+DllMain (HINSTANCE hinstDLL,
+	 DWORD     fdwReason,
+	 LPVOID    lpvReserved)
+{
+  wchar_t wcbfr[1000];
+  char cpbfr[1000];
+  char *dll_name = NULL;
+  gchar *prefix;
+  
+  switch (fdwReason) {
+  case DLL_PROCESS_ATTACH:
+	  /* GLib 2.6 uses UTF-8 file names */
+	  if (GetVersion () < 0x80000000) {
+		  /* NT-based Windows has wide char API */
+		  if (GetModuleFileNameW ((HMODULE) hinstDLL,
+					  wcbfr, G_N_ELEMENTS (wcbfr)))
+			  dll_name = g_utf16_to_utf8 (wcbfr, -1,
+						      NULL, NULL, NULL);
+	  } else {
+		  /* Win9x, yecch */
+		  if (GetModuleFileNameA ((HMODULE) hinstDLL,
+					  cpbfr, G_N_ELEMENTS (cpbfr)))
+			  dll_name = g_locale_to_utf8 (cpbfr, -1,
+						       NULL, NULL, NULL);
+	  }
+
+	  if (dll_name) {
+		  gchar *p = strrchr (dll_name, '\\');
+		  
+		  if (p != NULL)
+			  *p = '\0';
+
+		  p = strrchr (dll_name, '\\');
+		  if (p && (g_ascii_strcasecmp (p + 1, "bin") == 0))
+			  *p = '\0';
+		  
+		  prefix = dll_name;
+	  } else {
+		  prefix = g_strdup ("");
+	  }
+
+	  ORBit_win32_typelib_dir = g_strconcat (prefix,
+						 "\\lib\\orbit-2.0",
+						 NULL);
+	  ORBit_win32_system_rcfile = g_strconcat (prefix,
+						   "\\etc\\orbitrc",
+						   NULL);
+	  g_free (prefix);
+	  break;
+  }
+
+  return TRUE;
+}
+
+#endif
